from slippi.util import *
#－ーー


MAPPING = {
    # Hiragana
    0x82a0: 'あ',
    0x82a9: 'か',
    0x82b3: 'さ',
    0x82bd: 'た',
    0x82c8: 'な',
    0x82cd: 'は',
    0x82dc: 'ま',
    0x82e2: 'や',
    0x82e7: 'ら',
    0x82ed: 'わ',
    0x815b: 'ー',
    0x82a2: 'い',
    0x82ab: 'き',
    0x82b5: 'し',
    0x82bf: 'ち',
    0x82c9: 'に',
    0x82d0: 'ひ',
    0x82dd: 'み',
    0x82e4: 'ゆ',
    0x82e8: 'り',
    0x82f0: 'を',
    0x82a4: 'う',
    0x82ad: 'く',
    0x82b7: 'す',
    0x82c2: 'つ',
    0x82ca: 'ぬ',
    0x82d3: 'ふ',
    0x82de: 'む',
    0x82e6: 'よ',
    0x82e9: 'る',
    0x82f1: 'ん',
    0x8141: '。',
    0x82a6: 'え',
    0x82af: 'け',
    0x82b9: 'せ',
    0x82c4: 'て',
    0x82cb: 'ね',
    0x82d6: 'へ',
    0x82df: 'め',
    0x82e1: 'ゃ',
    0x82ea: 'れ',
    0x82ec: 'ゎ',
    0x8142: '、',
    0x82a8: 'お',
    0x82b1: 'こ',
    0x82bb: 'そ',
    0x82c6: 'と',
    0x82cc: 'の',
    0x82d9: 'ほ',
    0x82e0: 'も',
    0x82e3: 'ゅ',
    0x82eb: 'ろ',
    0x8143: '，',
    0x829f: 'ぁ',
    0x82aa: 'が',
    0x82b4: 'ざ',
    0x82be: 'だ',
    0x82ce: 'ば',
    0x82e5: 'ょ',
    0x8144: '．',
    0x82a1: 'ぃ',
    0x82ac: 'ぎ',
    0x82b6: 'じ',
    0x82c0: 'ぢ',
    0x82d1: 'び',
    0x8145: '・',
    0x82a3: 'ぅ',
    0x82ae: 'ぐ',
    0x82b8: 'ず',
    0x82c3: 'づ',
    0x82d4: 'ぶ',
    0x8146: '：',
    0x82a5: 'ぇ',
    0x82b0: 'げ',
    0x82ba: 'ぜ',
    0x82c5: 'で',
    0x82d7: 'べ',
    0x8147: '；',
    0x82a8: 'ぉ',
    0x82b2: 'ご',
    0x82bc: 'ぞ',
    0x82c7: 'ど',
    0x82da: 'ぼ',
    0x8148: '？',
    0x82c1: 'っ',
    0x82cf: 'ぱ',
    0x8149: '！',
    0x82d2: 'ぴ',
    0x82d5: 'ぷ',
    0x82d8: 'ぺ',
    0x82db: 'ぽ',

    # Katakana
    0x8341: 'ア',
    0x834a: 'カ',
    0x8354: 'サ',
    0x835e: 'タ',
    0x8369: 'ナ',
    0x836e: 'ハ',
    0x837d: 'マ',
    0x8384: 'ヤ',
    0x8389: 'ラ',
    0x838f: 'ワ',
    0x815b: 'ー',
    0x8343: 'イ',
    0x834c: 'キ',
    0x8356: 'シ',
    0x8360: 'チ',
    0x836a: 'ニ',
    0x8371: 'ヒ',
    0x837e: 'ミ',
    0x8386: 'ユ',
    0x838a: 'リ',
    0x8392: 'ヲ',
    0x8345: 'ウ',
    0x834e: 'ク',
    0x8358: 'ス',
    0x8363: 'ツ',
    0x836b: 'ヌ',
    0x8374: 'フ',
    0x8380: 'ム',
    0x8388: 'ヨ',
    0x838b: 'ル',
    0x8393: 'ン',
    0x8141: '。',
    0x8347: 'エ',
    0x8350: 'ケ',
    0x835a: 'セ',
    0x8365: 'テ',
    0x836c: 'ネ',
    0x8377: 'ヘ',
    0x8381: 'メ',
    0x8383: 'ャ',
    0x838c: 'レ',
    0x838e: 'ヮ',
    0x8142: '、',
    0x8349: 'オ',
    0x8352: 'コ',
    0x835c: 'ソ',
    0x8367: 'ト',
    0x836d: 'ノ',
    0x837a: 'ホ',
    0x8382: 'モ',
    0x8385: 'ュ',
    0x838d: 'ロ',
    0x8143: '，',
    0x8340: 'ァ',
    0x834b: 'ガ',
    0x8355: 'ザ',
    0x835f: 'ダ',
    0x836f: 'バ',
    0x8387: 'ョ',
    0x8144: '．',
    0x8342: 'ィ',
    0x834d: 'ギ',
    0x8357: 'ジ',
    0x8361: 'ヂ',
    0x8372: 'ビ',
    0x8145: '・',
    0x8344: 'ゥ',
    0x834f: 'グ',
    0x8359: 'ズ',
    0x8364: 'ヅ',
    0x8375: 'ブ',
    0x8146: '：',
    0x8346: 'ェ',
    0x8351: 'ゲ',
    0x835b: 'ゼ',
    0x8366: 'デ',
    0x8378: 'ベ',
    0x8147: '；',
    0x8348: 'ォ',
    0x8353: 'ゴ',
    0x835d: 'ゾ',
    0x8368: 'ド',
    0x837b: 'ボ',
    0x8148: '？',
    0x8394: 'ヴ',
    0x8395: 'ヵ',
    0x8362: 'ッ',
    0x8370: 'パ',
    0x8149: '！',
    0x8396: 'ヶ',
    0x8373: 'ピ',
    0x8376: 'プ',
    0x8379: 'ペ',
    0x837c: 'ポ',

    # Latin
    0x41: 'A',
    0x61: 'a',
    0x8260: 'Ａ',
    0x8281: 'ａ',
    0x42: 'B',
    0x62: 'b',
    0x8261: 'B',
    0x8282: 'ｂ',
    0x43: 'C',
    0x63: 'c',
    0x8262: 'Ｃ',
    0x8283: 'ｃ',
    0x44: 'D',
    0x64: 'd',
    0x8263: 'Ｄ',
    0x8284: 'ｄ',
    0x45: 'E',
    0x65: 'e',
    0x8264: 'Ｅ',
    0x8285: 'ｅ',
    0x46: 'F',
    0x66: 'f',
    0x8265: 'F',
    0x8286: 'ｆ',
    0x47: 'G',
    0x67: 'g',
    0x8266: 'G',
    0x8287: 'ｇ',
    0x48: 'H',
    0x68: 'h',
    0x8267: 'H',
    0x8288: 'ｈ',
    0x49: 'I',
    0x69: 'i',
    0x8268: 'I',
    0x8289: 'ｉ',
    0x4A: 'J',
    0x6a: 'j',
    0x8269: 'Ｈ',
    0x828a: 'ｊ',
    0x4B: 'K',
    0x6b: 'k',
    0x826a: 'Ｋ',
    0x828b: 'ｋ',
    0x4C: 'L',
    0x6c: 'l',
    0x826b: 'Ｌ',
    0x828c: 'ｌ',
    0x4D: 'M',
    0x6d: 'm',
    0x826c: 'Ｍ',
    0x828d: 'ｍ',
    0x4E: 'N',
    0x6e: 'n',
    0x826d: 'Ｎ',
    0x828e: 'ｎ',
    0x4F: 'O',
    0x6f: 'o',
    0x826e: 'Ｏ',
    0x828f: 'ｏ',
    0x50: 'P',
    0x70: 'p',
    0x826f: 'Ｐ',
    0x8290: 'ｐ',
    0x51: 'Q',
    0x71: 'q',
    0x8270: 'Ｑ',
    0x8291: 'ｑ',
    0x52: 'R',
    0x72: 'r',
    0x8271: 'Ｒ',
    0x8292: 'ｒ',
    0x53: 'S',
    0x73: 's',
    0x8272: 'Ｓ',
    0x8293: 'ｓ',
    0x54: 'T',
    0x74: 't',
    0x8273: 'Ｔ',
    0x8294: 'ｔ',
    0x55: 'U',
    0x75: 'u',
    0x8274: 'Ｕ',
    0x8295: 'ｕ',
    0x56: 'V',
    0x76: 'v',
    0x8275: 'Ｖ',
    0x8296: 'ｖ',
    0x57: 'W',
    0x77: 'w',
    0x8276: 'Ｗ',
    0x8297: 'ｗ',
    0x58: 'X',
    0x78: 'x',
    0x8277: 'Ｘ',
    0x8298: 'ｘ',
    0x59: 'Y',
    0x79: 'y',
    0x8278: 'Y',
    0x8299: 'ｙ',
    0x5a: 'Z',
    0x7a: 'z',
    0x8279: 'Ｚ',
    0x829a: 'ｚ',

    # Numbers
    0x30: '0',
    0x824f: '０',
    0x31: '1',
    0x8250: '１',
    0x32: '2',
    0x8251: '２',
    0x33: '3',
    0x8252: '３',
    0x34: '4',
    0x8253: '４',
    0x35: '5',
    0x8254: '５',
    0x36: '6',
    0x8255: '６',
    0x37: '7',
    0x8256: '７',
    0x38: '8',
    0x8257: '８',
    0x39: '9',
    0x8258: '９',

    # Symbols
    0x20: ' ',
    0x8141: '。',
    0x814F: '＾',
    0x8166: '’',
    0x817B: '＋',
    0x22: '"',
    0x8142: '、',
    0x8151: '＿',
    0x8168: '”',
    0x817C: '−',
    0x27: '\'',
    0x8143: '，',
    0x815B: 'ー',
    0x8169: '（',
    0x817E: '×',
    0x2c: ',',
    0x8144: '．',
    0x815E: '／',
    0x816A: '）',
    0x8181: '＝',
    0x2d: '-',
    0x8145: '・',
    0x8160: '〜',
    0x816D: '［',
    0x8183: '＜',
    0x2e: '.',
    0x8146: '：',
    0x8162: '｜',
    0x816E: '］',
    0x8184: '＞',
    0x3a: ':',
    0x8147: '；',
    0x816F: '｛',
    0x818F: '￥',
    0x8148: '？',
    0x8170: '｝',
    0x8190: '＄',
    0x8149: '！',
    0x8193: '％',
    0x8194: '＃',
    0x8195: '＆',
    0x8196: '＊',
    0x8197: '＠',

    0x0: ''
}


class String:
    """Melee stores strings (like nametags) in a custom 2-byte encoding."""

    def decode(melee_chars):
        print(melee_chars)
        return ''.join(MAPPING.get(c, '?') for c in melee_chars)
